/*
 * $Header: /home/cvspublic/jakarta-commons-sandbox/resources/src/java/org/apache/commons/resources/Messages.java,v 1.2 2002/12/28 23:29:48 craigmcc Exp $
 * $Revision: 1.2 $
 * $Date: 2002/12/28 23:29:48 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */

package org.apache.commons.resources;


import java.io.Serializable;
import java.text.MessageFormat;
import java.util.Locale;
import org.apache.commons.resources.Resources;
import org.apache.commons.resources.ResourcesException;
import org.apache.commons.resources.ResourcesFactory;
import org.apache.commons.resources.impl.ResourceBundleResourcesFactory;


/**
 * <p>Wrapper around any {@link Resources} object that performs message
 * string lookups from the {@link Resources} instance, and parameter
 * replacement via <code>java.text.MessageFormat</code>.  For convenience,
 * the same functionality is also available via static methods that accept
 * a {@link Resources} parameter.</p>
 *
 * <p>Calls to <code>getMessage()</code> variants without a <code>Locale</code>
 * argument are presumed to be requesting a message string in the default
 * <code>Locale</code> for this JVM.</p>
 *
 * <p>When a <code>getString()</code> call to the underlying {@link Resources}
 * instance fails or returns null, a suitable error message String will be
 * returned.</p>
 *
 * @author Craig R. McClanahan
 * @version $Revision: 1.2 $ $Date: 2002/12/28 23:29:48 $
 */

public class Messages implements Serializable {


    // ----------------------------------------------------------- Constructors


    /**
     * <p>Construct a {@link Messages} instance that wraps the specified
     * {@link Resources} instance.</p>
     *
     * @param resources The {@link Resources} instance from which
     *  message strings are to be retrieved
     */
    public Messages(Resources resources) {

        this.resources = resources;

    }


    // ----------------------------------------------------- Instance Variables


    /**
     * <p>The default <code>Locale</code> for this JVM.</p>
     */
    protected Locale defaultLocale = null;


    /**
     * <p>The {@link Resources} instance that we are wrapping.</p>
     */
    protected Resources resources = null;


    // ------------------------------------------------------------- Properties


    /**
     * <p>Return the {@link Resources} instance that we are wrapping.</p>
     */
    public Resources getResources() {

        return (this.resources);

    }


    // --------------------------------------------------------- Public Methods


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>.</p>
     *
     * @param key Message key to retrieve
     */
    public String getMessage(String key) {

        return (getMessage(resources, key));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>.</p>
     *
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     */
    public String getMessage(Locale locale, String key) {

        return (getMessage(resources, locale, key));

    }


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param key Message key to retrieve
     * @param args Array of replacement values
     */
    public String getMessage(String key, Object args[]) {

        return (getMessage(resources, key, args));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param args Array of replacement values
     */
    public String getMessage(Locale locale, String key, Object args[]) {

        return (getMessage(resources, locale, key, args));

    }


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     */
    public String getMessage(String key, Object arg0) {

        return (getMessage(resources, key, arg0));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     */
    public String getMessage(Locale locale, String key, Object arg0) {

        return (getMessage(resources, locale, key, arg0));

    }


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     */
    public String getMessage(String key, Object arg0, Object arg1) {

        return (getMessage(resources, key, arg0, arg1));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     */
    public String getMessage(Locale locale, String key, Object arg0,
                             Object arg1) {

        return (getMessage(resources, locale, key, arg0, arg1));

    }


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     * @param arg2 Individual parameter replacement value
     */
    public String getMessage(String key, Object arg0, Object arg1,
                             Object arg2) {

        return (getMessage(resources, key, arg0, arg1, arg2));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     * @param arg2 Individual parameter replacement value
     */
    public String getMessage(Locale locale, String key, Object arg0,
                             Object arg1, Object arg2) {

        return (getMessage(resources, locale, key, arg0, arg1, arg2));

    }


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     * @param arg2 Individual parameter replacement value
     * @param arg3 Individual parameter replacement value
     */
    public String getMessage(String key, Object arg0, Object arg1,
                             Object arg2, Object arg3) {

        return (getMessage(resources, key, arg0, arg1, arg2, arg3));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     * @param arg2 Individual parameter replacement value
     * @param arg3 Individual parameter replacement value
     */
    public String getMessage(Locale locale, String key, Object arg0,
                             Object arg1, Object arg2, Object arg3) {

        return (getMessage(resources, locale, key, arg0, arg1, arg2, arg3));

    }


    // ------------------------------------------------------- Static Variables


    /**
     * <p>The {@link ResourcesFactory} that will be used by the
     * <code>getMessages()</code> method.</p>
     */
    protected static ResourcesFactory factory = null;


    // --------------------------------------------------------- Static Methods


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param key Message key to retrieve
     */
    public static String getMessage(Resources resources, String key) {

        return (getMessage(resources, (Locale) null, key));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     */
    public static String getMessage(Resources resources, Locale locale,
                                    String key) {

        String message = null;
        try {
            message = resources.getString(key, locale, null);
        } catch (ResourcesException e) {
            ;
        }
        if (message == null) {
            message = "???" + key + "???";
        }
        return (message);

    }


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param key Message key to retrieve
     * @param args Array of replacement values
     */
    public static String getMessage(Resources resources, String key,
                                    Object args[]) {

        return (getMessage(resources, (Locale) null, key, args));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param args Array of replacement values
     */
    public static String getMessage(Resources resources, Locale locale,
                                    String key, Object args[]) {

        // FIXME - Cache MessageFormat instances?
        String message = getMessage(resources, locale, key);
	MessageFormat format = new MessageFormat(message);
        return (format.format(args));

    }


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     */
    public static String getMessage(Resources resources,
                                    String key, Object arg0) {

        return (getMessage(resources, (Locale) null, key, arg0));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     */
    public static String getMessage(Resources resources, Locale locale,
                                    String key, Object arg0) {

        Object args[] = new Object[1];
        args[0] = arg0;
        return (getMessage(resources, locale, key, args));

    }


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     */
    public static String getMessage(Resources resources,
                                    String key, Object arg0, Object arg1) {

        return (getMessage(resources, (Locale) null, key, arg0, arg1));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     */
    public static String getMessage(Resources resources, Locale locale,
                                    String key, Object arg0, Object arg1) {

        Object args[] = new Object[2];
        args[0] = arg0;
        args[1] = arg1;
        return (getMessage(resources, locale, key, args));

    }


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     * @param arg2 Individual parameter replacement value
     */
    public static String getMessage(Resources resources,
                                    String key, Object arg0, Object arg1,
                                    Object arg2) {

        return (getMessage(resources, (Locale) null, key, arg0, arg1, arg2));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     * @param arg2 Individual parameter replacement value
     */
    public static String getMessage(Resources resources, Locale locale,
                                    String key, Object arg0, Object arg1,
                                    Object arg2) {

        Object args[] = new Object[3];
        args[0] = arg0;
        args[1] = arg1;
        args[2] = arg2;
        return (getMessage(resources, locale, key, args));

    }


    /**
     * <p>Return a text message for the specified key, for the default
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     * @param arg2 Individual parameter replacement value
     * @param arg3 Individual parameter replacement value
     */
    public static String getMessage(Resources resources,
                                    String key, Object arg0, Object arg1,
                                    Object arg2, Object arg3) {

        return (getMessage(resources, (Locale) null, key, arg0, arg1,
                           arg2, arg3));

    }


    /**
     * <p>Return a text message for the specified key, for the specified
     * <code>Locale</code>, with parametric replacement.</p>
     *
     * @param resources {@link Resources} instance to retrieve the message from
     * @param locale <code>Locale</code> for which to retrieve the message
     * @param key Message key to retrieve
     * @param arg0 Individual parameter replacement value
     * @param arg1 Individual parameter replacement value
     * @param arg2 Individual parameter replacement value
     * @param arg3 Individual parameter replacement value
     */
    public static String getMessage(Resources resources, Locale locale,
                                    String key, Object arg0, Object arg1,
                                    Object arg2, Object arg3) {

        Object args[] = new Object[4];
        args[0] = arg0;
        args[1] = arg1;
        args[2] = arg2;
        args[3] = arg3;
        return (getMessage(resources, locale, key, args));

    }


    /**
     * <p>Convenience factory method to create a {@link Messages} instance
     * that wraps a {@link Resources} instance that contains message resources
     * for the specified Java package.  It is expected that the resources
     * for each package will be in properties files that are nested in the
     * package directory, with the name <code>LocalStrings.properties</code>
     * for the default messages, and names like
     * <code>LocalStrings_en_US.properties</code> for messages localized to
     * a particular <code>Locale</code>.</p>
     *
     * @param name Package name of the Java package for which
     *  local message resources are desired
     */
    public static Messages getMessages(String name) {

        if (factory == null) {
            factory = new ResourceBundleResourcesFactory();
        }
        try {
            Resources resources = factory.getResources(name + ".LocalStrings");
            return (new Messages(resources));
        } catch (ResourcesException e) {
            return (null);
        }

    }


}
