/*
 * $Header: /home/cvspublic/jakarta-commons-sandbox/resources/src/test/org/apache/commons/resources/impl/CollectionResourcesBaseTestCase.java,v 1.3 2003/01/14 00:08:45 jmitchell Exp $
 * $Revision: 1.3 $
 * $Date: 2003/01/14 00:08:45 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */


package org.apache.commons.resources.impl;


import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.apache.commons.resources.Resources;
import org.apache.commons.resources.ResourcesException;
import org.apache.commons.resources.ResourcesFactory;
import org.apache.commons.resources.ResourcesKeyException;
import org.apache.commons.resources.impl.CollectionResourcesBase;



/**
 * <p>Unit tests for
 * <code>org.apache.commons.resources.impl.CollectionResourcesBase</code>.
 * </p>
 *
 * @author Craig R. McClanahan
 * @version $Revision: 1.3 $ $Date: 2003/01/14 00:08:45 $
 */

public class CollectionResourcesBaseTestCase extends ResourcesBaseTestCase {


    // ----------------------------------------------------- Instance Variables


    // Base URL of the resource files for the Resources instance to be created
    protected static final String BASE = "fixme";


    // ----------------------------------------------------------- Constructors


    public CollectionResourcesBaseTestCase(String name) {
        super(name);
    }


    // --------------------------------------------------- Overall Test Methods


    // Set up instance variables required by this test case
    public void setUp() throws Exception {
        factory = new CollResourcesFactory();
        resources = factory.getResources(NAME, BASE);
    }

    // Return the tests included in this test suite
    public static Test suite() {
        return (new TestSuite(CollectionResourcesBaseTestCase.class));
    }

    // Tear down the instance variables required by this test case
    public void tearDown() {
        resources = null;
        factory = null;
    }


    // ------------------------------------------------ Individual Test Methods


    // Test retrieving Strings that should be inherited from parent Locales
    public void testInherit() throws Exception {

        Locale locale = null;

        locale = new Locale("en", "US");
        assertEquals("en_US inherit value",
                     "[en] INHERIT",
                     resources.getString("test.inherit", locale, null));

        locale = new Locale("fr", "FR");
        assertEquals("fr_FR inherit value",
                     "[fr] INHERIT",
                     resources.getString("test.inherit", locale, null));

    }


    // Test the getLocaleList() method
    public void testLocaleList() {

        Locale test = null;
        List expected = null;
        CollectionResourcesBase cresources =
            (CollectionResourcesBase) resources;

        test = new Locale("en", "US", "POSIX");
        expected = new ArrayList();
        expected.add(test);
        expected.add(new Locale("en", "US"));
        expected.add(new Locale("en", ""));
        expected.add(new Locale("", ""));
        checkLocaleList(test, expected, cresources.getLocaleList(test));

        test = new Locale("en", "US");
        expected = new ArrayList();
        expected.add(test);
        expected.add(new Locale("en", ""));
        expected.add(new Locale("", ""));
        checkLocaleList(test, expected, cresources.getLocaleList(test));

        test = new Locale("en", "");
        expected = new ArrayList();
        expected.add(test);
        expected.add(new Locale("",""));
        checkLocaleList(test, expected, cresources.getLocaleList(test));

        test = new Locale("", "US", "POSIX");
        expected = new ArrayList();
        expected.add(test);
        expected.add(new Locale("", "US"));
        expected.add(new Locale("", ""));
        checkLocaleList(test, expected, cresources.getLocaleList(test));

        test = new Locale("", "US");
        expected = new ArrayList();
        expected.add(test);
        expected.add(new Locale("", ""));
        checkLocaleList(test, expected, cresources.getLocaleList(test));

    }


    // Test retrieving missing resource keys
    public void testMissing() throws Exception {

        Locale locale = new Locale("en","");
        try {
            resources.setReturnNull(true);
            String value = resources.getString("test.missing", locale, null);
            assertTrue("Should have returned null", value == null);
        } catch (ResourcesException e) {
            fail("Should have returned null");
        }
        try {
            resources.setReturnNull(false); 
           String value = resources.getString("test.missing", locale, null);
            fail("Should have thrown ResourcesException");
        } catch (ResourcesKeyException e) {
            ; // Expected result
        }

    }


    // Test retrieving Locale-specific Strings explicitly
    public void testSpecific() throws Exception {

        Locale locale = null;

        locale = new Locale("en", "");
        assertEquals("en specific value",
                     "[en] SPECIFIC",
                     resources.getString("test.specific", locale, null));

        locale = new Locale("en", "US");
        assertEquals("en_US specific value",
                     "[en_US] SPECIFIC",
                     resources.getString("test.specific", locale, null));

        locale = new Locale("fr", "");
        assertEquals("fr specific value",
                     "[fr] SPECIFIC",
                     resources.getString("test.specific", locale, null));

    }


    // ------------------------------------------------------ Protected Methods


    // Check the results of a getLocaleList() operation
    protected void checkLocaleList(Locale test, List expected, List actual) {

        assertEquals("[" + test + "] list length",
                     expected.size(),
                     actual.size());
        for (int i = 0; i < expected.size(); i++) {
            assertEquals("[" + test + "] list contents[" + i + "]",
                         expected.get(i),
                         actual.get(i));
        }

    }


}
